# frozen_string_literal: true

Dir[File.dirname(__FILE__)].each { |file| $LOAD_PATH.unshift(file) if File.directory? file }

require 'dotenv/load'
require 'druuid'
require 'fiber'
require 'logger'
require 'pastel'
require 'pry'
require 'set'

module RuneRb
  VERSION = '0.0.2'

  # RuneRb::System
  module System
    autoload :Controller,                 'system/controller'
    autoload :Constants,                  'system/constants'

    # RuneRb::Types - Common types used in the RuneRb framework.
    module Types
      autoload :Controller,               'system/types/controller'
      autoload :SortedStack,              'system/types/stack'
      autoload :Service,                  'system/types/service'
      autoload :Task,                     'system/types/task'
    end

    # RuneRb::System::Utils - Utility modules and interfaces.
    module Utils
      autoload :Logging,                  'system/utils/logging'
      autoload :Pipeline,                 'system/utils/pipeline'
    end

    # RuneRb::System::Patches - Core/Stdlib class and module patches.
    module Patches
      autoload :IntegerRefinements,       'system/patches/integer'
      autoload :SetRefinements,           'system/patches/set'
      autoload :StringRefinements,        'system/patches/string'
    end

    # RuneRb::System::Errors - Errors triggered and handled exclusively by the framework or it's API.
    module Errors
      autoload :UnrecognizedMessage,      'system/errors'
      autoload :SessionReceptionError,    'system/errors'
      autoload :ConflictingNameError,     'system/errors'
    end

    include Constants
  end
end

# Copyright (c) 2021, Patrick W.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
