# frozen_string_literal: true

module RuneRb::System::Constants
  # @!attribute [r] TASK_PRIORITIES
  # @return [Hash] a map of priorities for RuneRb::System::Types::Task objects.
  TASK_PRIORITIES = { LOW: 2, MEDIUM: 1, HIGH: 0 }.freeze

  # @!attribute [r] COLORS
  # @return [Pastel] a color object used for colorful logging.
  COLORS = Pastel.new(enabled: ENV['RRB_LOG_COLOR_ALL'].to_i.nonzero?, eachline: ENV['RRB_LOG_COLOR_EACH'].to_i.nonzero?)

  # @!attribute [r] LOGGER
  # @return [Logger] a Logger used for logging.
  LOGGER = Logger.new($stdout)
  FileUtils.mkdir_p('assets/log') unless File.exist?('assets/log') && File.directory?('assets/log')
  LOGGER.formatter = proc do |sev, date, _prog, msg|
    "#{COLORS.green.bold("[#{date.strftime('%H:%M')}]")}|#{COLORS.blue.bold("[#{sev}]")} -> #{msg}\n"
  end

  # @!attribute [r] LOG_FILE
  # @return [Logger] a Logger used for file-based logging.
  LOG_FILE = Logger.new("#{ENV['RRB_LOG_PATH'].nil? ? 'assets/log' : ENV['RRB_LOG_PATH']}/rune.rb-#{Time.now.strftime('%Y-%m-%d').chomp}.log",
                        progname: "rune.rb-core-#{RuneRb::VERSION}")
  LOG_FILE.formatter = proc do |sev, _date, prog, msg|
    "[#{prog}]|[#{sev}] -> #{msg}\n"
  end
end

# Copyright (c) 2021, Patrick W.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.